@@ -2,13 +2,15 @@ use crate::storage::{ResourceStorage, Stream};
 
 use memmap::Mmap;
 
-use std::cell::RefCell;
-use std::collections::BTreeMap;
-use std::fs::{self, File};
-use std::io;
-use std::path::PathBuf;
-use std::rc::Rc;
-use std::slice;
+use std::{
+    cell::RefCell,
+    collections::BTreeMap,
+    fs::{self, File},
+    io,
+    path::PathBuf,
+    rc::Rc,
+    slice,
+};
 
 /// Internal storage of data as files.
 #[derive(Debug, Default)]
@@ -31,9 +33,24 @@ impl MemoryMappedFileStorage {
     }
 }
 
-impl Stream for File {}
-
 /// Resource storage on disk using memory mapped files.
+///
+/// Can be used to create and read archives from the file system.
+///
+/// # Examples
+/// ``` ignore
+/// # #[macro_use] extern crate flatdata;
+/// # fn main() {
+/// # use flatdata::{ FileResourceStorage, Archive, ArchiveBuilder };
+/// # define_archive!(X, XBuilder, "Schema for X"; );
+/// let storage = FileResourceStorage::new(PathBuf::from("/root/to/my/archive"));
+/// let _builder = XBuilder::new(storage.clone()).expect("failed to create builder");
+/// // Write some data, store archive, etc...
+///
+/// let _archive = X::open(storage).expect("failed to open");
+/// // read some data
+/// # }
+/// ```
 #[derive(Debug)]
 pub struct FileResourceStorage {
     storage: MemoryMappedFileStorage,
@@ -42,7 +59,6 @@ pub struct FileResourceStorage {
 
 impl FileResourceStorage {
     /// Create an empty memory mapped file storage at a given path.
-    #[allow(clippy::new_ret_no_self)]
     pub fn new<P: Into<PathBuf>>(path: P) -> Rc<Self> {
         Rc::new(Self {
             storage: MemoryMappedFileStorage::default(),
