@@ -1,36 +1,63 @@
 use crate::storage::{ResourceStorage, Stream};
 
-use std::cell::RefCell;
-use std::collections::BTreeMap;
-use std::fmt;
-use std::io::{self, Cursor};
-use std::path::PathBuf;
-use std::rc::Rc;
-use std::slice;
+use std::{
+    cell::RefCell,
+    collections::BTreeMap,
+    fmt,
+    io::{self, Cursor},
+    path::PathBuf,
+    rc::Rc,
+    slice,
+};
 
 type MemoryStorageStream = Rc<RefCell<Cursor<Vec<u8>>>>;
 
 /// Internal storage of data in memory.
-#[derive(Default)]
+#[derive(Default, Clone)]
 struct MemoryStorage {
     // Streams of resources that were written.
-    streams: RefCell<BTreeMap<PathBuf, MemoryStorageStream>>,
+    streams: Rc<RefCell<BTreeMap<PathBuf, MemoryStorageStream>>>,
     // Data of resources that were opened for reading.
-    resources: RefCell<BTreeMap<PathBuf, Rc<Vec<u8>>>>,
+    resources: Rc<RefCell<BTreeMap<PathBuf, Rc<Vec<u8>>>>>,
 }
 
 impl fmt::Debug for MemoryStorage {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         write!(
             f,
-            "MemoryStorage {{ num_streams: {}, num_resources: {} }}",
-            self.streams.borrow().len(),
-            self.resources.borrow().len(),
+            "MemoryStorage {{ streams: {:?}, resources: {:?} }}",
+            self.streams
+                .borrow()
+                .iter()
+                .map(|(path, _)| path.display())
+                .collect::<Vec<_>>(),
+            self.resources
+                .borrow()
+                .iter()
+                .map(|(path, _)| path.display())
+                .collect::<Vec<_>>(),
         )
     }
 }
 
 /// Resource storage in memory.
+///
+/// Can be used to create archives in-memory, e.g. when writing tests.
+///
+/// # Examples
+/// ```
+/// # #[macro_use] extern crate flatdata;
+/// # fn main() {
+/// # use flatdata::{ MemoryResourceStorage, Archive, ArchiveBuilder };
+/// # define_archive!(X, XBuilder, "Schema for X"; );
+/// let storage = MemoryResourceStorage::new("/root/extvec");
+/// let _builder = XBuilder::new(storage.clone()).expect("failed to create builder");
+/// // Write some data
+///
+/// let _archive = X::open(storage).expect("failed to open");
+/// // read some data
+/// # }
+/// ```
 #[derive(Debug)]
 pub struct MemoryResourceStorage {
     storage: MemoryStorage,
@@ -51,11 +78,12 @@ impl MemoryResourceStorage {
     }
 }
 
-impl Stream for Cursor<Vec<u8>> {}
-
 impl ResourceStorage for MemoryResourceStorage {
     fn subdir(&self, dir: &str) -> Rc<ResourceStorage> {
-        Self::new(self.path.join(dir))
+        Rc::new(Self {
+            storage: self.storage.clone(),
+            path: self.path.join(dir),
+        })
     }
 
     fn exists(&self, resource_name: &str) -> bool {
